@@ -59,15 +59,15 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
  */
 
 char *
-DBPrintUseId(scx, name, size, display_only)
-    SearchContext *scx;	/* Pointer to current search context, specifying a
+DBPrintUseId(
+    SearchContext *scx,	/* Pointer to current search context, specifying a
 			 * cell use and X,Y array indices.
 			 */
-    char *name;		/* Pointer to string into which we will copy the
+    char *name,		/* Pointer to string into which we will copy the
 			 * print name of this instance.
 			 */
-    int size;		/* Maximum number of characters to copy into string. */
-    bool display_only;	/* TRUE if called for displaying only */
+    int size,		/* Maximum number of characters to copy into string. */
+    bool display_only)	/* TRUE if called for displaying only */
 {
     CellUse *use = scx->scx_use;
     char *sp, *id, *ep;
@@ -619,8 +619,10 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
     /*
      * Include area of subcells separately
      */
-    if ((foundAny = DBBoundCellPlane(cellDef, TRUE, &rect)) > 0)
+    if ((foundAny = DBBoundCellPlane(cellDef, &extended, &rect)) > 0)
 	area = rect;
+    else
+	extended = GeoNullRect;
 
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (pNum != PL_DRC_CHECK)
@@ -634,7 +636,7 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
 	    }
 
     /*
-     * Include the area of labels, too.
+     * Include the area of label anchors, too.
      */
     for (label = cellDef->cd_labels; label != NULL;  label = label->lab_next)
     {
@@ -656,7 +658,11 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
 	}
     }
 
-    extended = area;
+    /* Make sure the extended bounding box includes the area of all
+     * paint material just found, then include the area of all text
+     * in the current cell.
+     */
+    GeoInclude(&area, &extended);
     if (foundAny)
     {
 	for (label = cellDef->cd_labels; label != NULL;  label = label->lab_next)
@@ -673,6 +679,7 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
 	degenerate = TRUE;
 	area.r_xbot = area.r_ybot = 0;
 	area.r_xtop = area.r_ytop = 1;
+	extended = area;
     }
     else degenerate = FALSE;
 
@@ -687,7 +694,11 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
     if (area.r_ybot == area.r_ytop)
 	area.r_ytop = area.r_ybot + 1;
 
-    if (degenerate) extended = area;
+    if (extended.r_xbot == extended.r_xtop)
+	extended.r_xtop = extended.r_xbot + 1;
+
+    if (extended.r_ybot == extended.r_ytop)
+	extended.r_ytop = extended.r_ybot + 1;
 
     /* Did the bounding box change?  If not then there's no need to
      * recompute the parents.  If the cell has no material, then
