@@ -13,7 +13,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/ext2sim/ext2sim.c,v 1.2 2008/12/03 14:12:09 tim Exp $";
+static const char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/ext2sim/ext2sim.c,v 1.2 2008/12/03 14:12:09 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -45,16 +45,19 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "extflat/extflat.h"
 
 /* Forward declarations */
-void CmdExtToSim();
-bool simnAP();
-bool simnAPHier();
-int simParseArgs();
-int simdevVisit(), simresistVisit(), simcapVisit(), simnodeVisit();
-int simmergeVisit();
+void CmdExtToSim(MagWindow *w, TxCommand *cmd);
+bool simnAP(EFNode *node, int resClass, float scale, FILE *outf);
+bool simnAPHier(const DevTerm *dterm, const HierName *hierName, int resClass, float scale, FILE *outf);
+bool simParseArgs(int *pargc, char ***pargv, ClientData cdata); /* @typedef cb_extflat_args_t (UNUSED) */
+int simdevVisit(Dev *dev, HierContext *hc, float scale, Transform *trans, ClientData cdata); /* @typedef cb_extflat_visitdevs_t (UNUSED) */
+int simresistVisit(const HierName *hierName1, const HierName *hierName2, float res, ClientData cdata); /* @typedef cb_extflat_visitresists_t (UNUSED) */
+int simcapVisit(const HierName *hierName1, const HierName *hierName2, double cap, ClientData cdata); /* @typedef cb_extflat_visitcaps_t (UNUSED) */
+int simnodeVisit(EFNode *node, int res, double cap, ClientData cdata); /* @typedef cb_extflat_visitnodes_t (UNUSED) */
+int simmergeVisit(Dev *dev, HierContext *hc, float scale, Transform *trans, ClientData cdata); /* @typedef cb_extflat_visitdevs_t (UNUSED) */
 
 /* C99 compat */
-int simdevOutNode();
-int simdevSubstrate();
+int simdevOutNode(const HierName *prefix, const HierName *suffix, const char *name, FILE *outf);
+int simdevSubstrate(const HierName *prefix, const HierName *suffix, int type, float scale, bool doAP, FILE *outf);
 
 /* Options specific to ext2sim */
 #ifdef EXT2SIM_AUTO
@@ -70,18 +73,18 @@ extern bool esNoAttrs;
 extern bool esHierAP;
 extern bool esMergeDevsA;
 extern bool esMergeDevsC;
-extern char esSpiceDefaultGnd[];
-extern char *esSpiceCapNode;
+extern const char esSpiceDefaultGnd[];
+extern const char *esSpiceCapNode;
 #endif
 
 bool esDoSimExtResis = FALSE;
 bool esNoAlias = TRUE;
 bool esNoLabel = TRUE;
 char simesDefaultOut[FNSIZE];
-char *simesOutName = simesDefaultOut;
+const char *simesOutName = simesDefaultOut;
 char esDefaultAlias[FNSIZE], esDefaultLabel[FNSIZE];
-char *esAliasName = esDefaultAlias;
-char *esLabelName = esDefaultLabel;
+const char *esAliasName = esDefaultAlias;
+const char *esLabelName = esDefaultLabel;
 char esCapFormat[FNSIZE];
 FILE *esSimF = NULL;
 FILE *esAliasF = NULL;
@@ -92,23 +95,23 @@ FILE *esLabF = NULL;
 #define	LBL 1
 #define	SU  2
 
-static unsigned short esFormat = MIT ;
+static unsigned short esFormat = MIT;
 
 struct {
-   short resClassSource ;   /* The resistance class of the source of the dev */
-   short resClassDrain ;    /* The resistance class of the drain of the dev */
-   short resClassSub ;	    /* The resistance class of the substrate of the dev */
-   TileType devType ;	    /* Magic tile type of the device */
-   char  *defSubs ;	    /* The default substrate node */
+    short resClassSource;   /* The resistance class of the source of the dev */
+    short resClassDrain;    /* The resistance class of the drain of the dev */
+    short resClassSub;	    /* The resistance class of the substrate of the dev */
+    TileType devType;	    /* Magic tile type of the device */
+    const char *defSubs;    /* The default substrate node */
 } fetInfo[TT_MAXTYPES];
 
 typedef struct {
-	TileTypeBitMask visitMask;
+    TileTypeBitMask visitMask;
 } nodeClient;
 
 typedef struct {
-	HierName *lastPrefix;
-	TileTypeBitMask visitMask;
+    const HierName *lastPrefix;
+    TileTypeBitMask visitMask;
 } nodeClientHier;
 
 #define NO_RESCLASS	-1
@@ -120,19 +123,19 @@ typedef struct {
   { TTMaskZero(&((client)->visitMask)); }
 
 #define beenVisited(client, rclass)  \
-  ( TTMaskHasType(&((client)->visitMask), rclass) )
+  (TTMaskHasType(&((client)->visitMask), rclass))
 
 #define initNodeClient(node) \
 { \
-	(node)->efnode_client = (ClientData) mallocMagic((unsigned) (sizeof(nodeClient))); \
-	TTMaskZero(&(( nodeClient *)(node)->efnode_client)->visitMask); \
+	(node)->efnode_client = PTR2CD(mallocMagic(sizeof(nodeClient))); \
+	TTMaskZero(&((nodeClient *)(node)->efnode_client)->visitMask); \
 }
 
 
 #define initNodeClientHier(node) \
 { \
-	(node)->efnode_client = (ClientData) mallocMagic((unsigned) (sizeof(nodeClientHier))); \
-	TTMaskZero(&(( nodeClientHier *)(node)->efnode_client)->visitMask); \
+	(node)->efnode_client = PTR2CD(mallocMagic(sizeof(nodeClientHier))); \
+	TTMaskZero(&((nodeClientHier *)(node)->efnode_client)->visitMask); \
 }
 
 
@@ -149,13 +152,13 @@ int  esDevsMerged;
 /* macro to add a dev's multiplier to the table and grow it if necessary */
 #define addDevMult(f) \
 {  \
-        if ( esFMult == NULL ) { \
+        if (esFMult == NULL) { \
           esFMult = (float *) mallocMagic((unsigned) (esFMSize*sizeof(float)));  \
-        } else if ( esFMIndex >= esFMSize ) {  \
+        } else if (esFMIndex >= esFMSize) {  \
           int i;  \
-          float *op = esFMult ;  \
+          float *op = esFMult;  \
           esFMult = (float *) mallocMagic((unsigned) ((esFMSize = esFMSize*2)*sizeof(float))); \
-          for ( i = 0 ; i < esFMSize/2 ; i++ ) esFMult[i] = op[i]; \
+          for (i = 0; i < esFMSize/2; i++) esFMult[i] = op[i]; \
           if (op) freeMagic(op); \
         }  \
         esFMult[esFMIndex++] = (float)(f); \
@@ -168,17 +171,17 @@ int  esDevsMerged;
 /* cache list used to find parallel devs */
 typedef struct _devMerge {
         int     l, w;
-        EFNode *g, *s, *d, *b;
+        const EFNode *g, *s, *d, *b;
         Dev * dev;
         int       esFMIndex;
-        HierName *hierName;
-        struct _devMerge *next;
+        const HierName *hierName;
+        const struct _devMerge *next;
 } devMerge;
 
 #ifdef EXT2SIM_AUTO
-devMerge *devMergeList = NULL ;
+const devMerge *devMergeList = NULL;
 #else
-extern devMerge *devMergeList;
+extern const devMerge *devMergeList;
 #endif
 
 /* attributes controlling the Area/Perimeter extraction of fet terminals */
@@ -190,9 +193,6 @@ extern devMerge *devMergeList;
 
 #ifdef MAGIC_WRAPPER
 
-/* Forward declaration */
-int _ext2sim_start();
-
 #ifdef EXT2SIM_AUTO
 /*
  * ----------------------------------------------------------------------------
@@ -203,8 +203,8 @@ int _ext2sim_start();
  */
 
 int
-Exttosim_Init(interp)
-    Tcl_Interp *interp;
+Exttosim_Init(
+    Tcl_Interp *interp)
 {
     /* Sanity checks! */
     if (interp == NULL) return TCL_ERROR;
@@ -252,13 +252,12 @@ Exttosim_Init(interp)
 #define EXTTOSIM_HELP		9
 
 void
-CmdExtToSim(w, cmd)
-    MagWindow *w;
-    TxCommand *cmd;
+CmdExtToSim(
+    MagWindow *w,
+    TxCommand *cmd)
 {
     int i,flatFlags;
-    char *inName;
-    FILE *f;
+    const char *inName;
 
     int value;
     int option = EXTTOSIM_RUN;
@@ -268,8 +267,8 @@ CmdExtToSim(w, cmd)
     bool err_result;
 
     short s_rclass, d_rclass, sub_rclass;
-    char *devname;
-    char *subname;
+    const char *devname;
+    const char *subname;
     TileType devtype;
     int idx;
 
@@ -497,12 +496,12 @@ CmdExtToSim(w, cmd)
 	    EFScale = 0.0;
 	    if (EFArgTech)
 	    {
-		freeMagic(EFArgTech);
+		freeMagic((char *) EFArgTech);
 		EFArgTech = NULL;
 	    }
 	    if (EFSearchPath)
 	    {
-		freeMagic(EFSearchPath);
+		freeMagic((char *) EFSearchPath);
 		EFSearchPath = NULL;
 	    }
 	    break;
@@ -531,7 +530,7 @@ runexttosim:
     EFResistThreshold = LocResistThreshold;
 
     /* Process command line arguments */
-    inName = EFArgs(argc, argv, &err_result, simParseArgs, (ClientData) NULL);
+    inName = EFArgs(argc, argv, &err_result, simParseArgs, PTR2CD(NULL));
 
     if (err_result == TRUE)
     {
@@ -631,7 +630,7 @@ runexttosim:
     /* create default fetinfo entries (MOSIS) which can be overriden by
        the command line arguments */
 
-    for ( i = 0 ; i < TT_MAXTYPES ; i++ )
+    for (i = 0; i < TT_MAXTYPES; i++)
     {
 	fetInfo[i].resClassSource = NO_RESCLASS;
 	fetInfo[i].resClassDrain = NO_RESCLASS;
@@ -669,7 +668,7 @@ runexttosim:
 
     fprintf(esSimF, "| units: %g tech: %s format: %s\n", EFScale, EFTech,
 		   (esFormat == MIT) ? "MIT" :
-		      ( (esFormat == LBL) ? "LBL" : "SU" ) );
+		      ((esFormat == LBL) ? "LBL" : "SU"));
 
     /* Convert the hierarchical description to a flat one */
     flatFlags = EF_FLATNODES;
@@ -679,23 +678,23 @@ runexttosim:
 
     if (esMergeDevsA || esMergeDevsC)
     {
-	devMerge *p;
+	const devMerge *p;
 
-	EFVisitDevs(simmergeVisit, (ClientData) NULL);
+	EFVisitDevs(simmergeVisit, PTR2CD(NULL));
 	TxPrintf("Devices merged: %d\n", esDevsMerged);
 	esFMIndex = 0;
 	for (p = devMergeList; p != NULL; p = p->next)
-	    freeMagic(p);
+	    freeMagic((char *)p);
 	devMergeList = NULL;
     }
 
-    EFVisitDevs(simdevVisit, (ClientData)NULL);
+    EFVisitDevs(simdevVisit, PTR2CD(NULL));
     if (flatFlags & EF_FLATCAPS) {
-	EFVisitCaps(simcapVisit, (ClientData) NULL);
+	EFVisitCaps(simcapVisit, PTR2CD(NULL));
     }
-    EFVisitResists(simresistVisit, (ClientData) NULL);
+    EFVisitResists(simresistVisit, PTR2CD(NULL));
     esSpiceCapNode = esSpiceDefaultGnd;
-    EFVisitNodes(simnodeVisit, (ClientData) NULL);
+    EFVisitNodes(simnodeVisit, PTR2CD(NULL));
 
     EFFlatDone(NULL);
     EFDone(NULL);
@@ -707,7 +706,7 @@ runexttosim:
     TxPrintf("exttosim finished.\n");
 }
 
-#if 0	/* Independent program "ext2sim" deprecated */
+#ifdef EXT2SIM_MAIN	/* Independent program "ext2sim" deprecated */
 
 /*
  * ----------------------------------------------------------------------------
@@ -720,13 +719,13 @@ runexttosim:
  */
 
 int
-main(argc, argv)
-    int argc;
-    char *argv[];
+main(
+    int argc,
+    char *argv[])
 {
 
     int i,flatFlags;
-    char *inName;
+    const char *inName;
     FILE *f;
 
     esDevsMerged = 0;
@@ -734,7 +733,7 @@ main(argc, argv)
     EFInit();
     /* create default fetinfo entries (MOSIS) which can be overriden by
        the command line arguments */
-    for ( i = 0 ; i < TT_MAXTYPES ; i++ ) {
+    for (i = 0; i < TT_MAXTYPES; i++) {
 	fetInfo[i].resClassSource = NO_RESCLASS;
 	fetInfo[i].resClassDrain = NO_RESCLASS;
 	fetInfo[i].resClassSub = NO_RESCLASS;
@@ -742,23 +741,23 @@ main(argc, argv)
 	fetInfo[i].devType = TT_SPACE;
     }
     i = efBuildAddStr(EFDevTypes, &EFDevNumTypes, TT_MAXTYPES, "nfet");
-    fetInfo[i].resClassSource = fetInfo[i].resClassDrain = 0 ;
-    fetInfo[i].resClassSub = NO_RESCLASS ;
+    fetInfo[i].resClassSource = fetInfo[i].resClassDrain = 0;
+    fetInfo[i].resClassSub = NO_RESCLASS;
     fetInfo[i].defSubs = "Gnd!";
     i = efBuildAddStr(EFDevTypes, &EFDevNumTypes, TT_MAXTYPES, "pfet");
-    fetInfo[i].resClassSource = fetInfo[i].resClassDrain = 1 ;
-    fetInfo[i].resClassSub = 6 ;
+    fetInfo[i].resClassSource = fetInfo[i].resClassDrain = 1;
+    fetInfo[i].resClassSub = 6;
     fetInfo[i].defSubs = "Vdd!";
     i = efBuildAddStr(EFDevTypes, &EFDevNumTypes, TT_MAXTYPES, "nmos");
-    fetInfo[i].resClassSource = fetInfo[i].resClassDrain = 0 ;
-    fetInfo[i].resClassSub = NO_RESCLASS ;
+    fetInfo[i].resClassSource = fetInfo[i].resClassDrain = 0;
+    fetInfo[i].resClassSub = NO_RESCLASS;
     fetInfo[i].defSubs = "Gnd!";
     i = efBuildAddStr(EFDevTypes, &EFDevNumTypes, TT_MAXTYPES, "pmos");
-    fetInfo[i].resClassSource = fetInfo[i].resClassDrain = 1 ;
-    fetInfo[i].resClassSub = 6 ;
+    fetInfo[i].resClassSource = fetInfo[i].resClassDrain = 1;
+    fetInfo[i].resClassSub = 6;
     fetInfo[i].defSubs = "Vdd!";
     /* Process command line arguments */
-    inName = EFArgs(argc, argv, NULL, simParseArgs, (ClientData) NULL);
+    inName = EFArgs(argc, argv, NULL, simParseArgs, PTR2CD(NULL));
     if (inName == NULL)
 	exit (1);
 
@@ -799,7 +798,7 @@ main(argc, argv)
 
     fprintf(esSimF, "| units: %g tech: %s format: %s\n", EFScale, EFTech,
 		   (esFormat == MIT) ? "MIT" :
-		      ( (esFormat == LBL) ? "LBL" : "SU" ) );
+		      ((esFormat == LBL) ? "LBL" : "SU"));
 
     /* Convert the hierarchical description to a flat one */
     flatFlags = EF_FLATNODES;
@@ -808,20 +807,20 @@ main(argc, argv)
     EFFlatBuild(inName, flatFlags);
 
     if (esMergeDevsA || esMergeDevsC) {
-	devMerge *p;
+	const devMerge *p;
 
-	EFVisitDevs(simmergeVisit, (ClientData) NULL);
+	EFVisitDevs(simmergeVisit, PTR2CD(NULL));
 	TxPrintf("Devices merged: %d\n", esDevsMerged);
 	esFMIndex = 0;
-	for (p = devMergeList; p != NULL; p = p->next) freeMagic(p);
+	for (p = devMergeList; p != NULL; p = p->next) freeMagic((char *)p);
     }
 
-    EFVisitDevs(simdevVisit, (ClientData) NULL);
+    EFVisitDevs(simdevVisit, PTR2CD(NULL));
     if (flatFlags & EF_FLATCAPS)
-	EFVisitCaps(simcapVisit, (ClientData) NULL);
-    EFVisitResists(simresistVisit, (ClientData) NULL);
+	EFVisitCaps(simcapVisit, PTR2CD(NULL));
+    EFVisitResists(simresistVisit, PTR2CD(NULL));
     esSpiceCapNode = esSpiceDefaultGnd;
-    EFVisitNodes(simnodeVisit, (ClientData) NULL);
+    EFVisitNodes(simnodeVisit, PTR2CD(NULL));
 
     EFFlatDone(NULL);
     EFDone(NULL);
@@ -834,7 +833,7 @@ main(argc, argv)
     exit(0);
 }
 
-#endif		/* Deprecated */
+#endif /* EXT2SIM_MAIN Deprecated */
 
 /*
  * ----------------------------------------------------------------------------
@@ -861,12 +860,14 @@ main(argc, argv)
  * ----------------------------------------------------------------------------
  */
 
-int
-simParseArgs(pargc, pargv)
-    int *pargc;
-    char ***pargv;
+/* @typedef cb_extflat_args_t */
+bool
+simParseArgs(
+    int *pargc,
+    char ***pargv,
+    ClientData cdata)	/* unused */
 {
-    char **argv = *pargv, *cp;
+    char **argv = *pargv;
     int argc = *pargc;
 
     switch (argv[0][1])
@@ -902,37 +903,37 @@ simParseArgs(pargc, pargv)
 		goto usage;
 	    break;
 	case 'f': {
-	     char *ftmp ;
+	     const char *ftmp;
 
 	     if ((ftmp = ArgStr(&argc, &argv, "format")) == NULL)
 		goto usage;
-	     if ( strcasecmp(ftmp,"MIT") == 0 )
-	        esFormat = MIT ;
-	     else if ( strcasecmp(ftmp,"LBL") == 0 )
-		esFormat = LBL ;
-	     else if ( strcasecmp(ftmp,"SU") == 0 )
-		esFormat = SU ;
+	     if (strcasecmp(ftmp,"MIT") == 0)
+	        esFormat = MIT;
+	     else if (strcasecmp(ftmp,"LBL") == 0)
+		esFormat = LBL;
+	     else if (strcasecmp(ftmp,"SU") == 0)
+		esFormat = SU;
 	     else goto usage;
 	     break;
 	     }
 	case 'y': {
-	      char *t;
+	      const char *t;
 
-	      if (( t =  ArgStr(&argc, &argv, "cap-accuracy") ) == NULL)
+	      if ((t = ArgStr(&argc, &argv, "cap-accuracy")) == NULL)
 		goto usage;
 	      TxPrintf("Cap accuracy option -y is deprecated.\n");
 	      break;
 	      }
 	case 'J':
 	     {
-	     char *ftmp ;
+	     const char *ftmp;
 
 	     if ((ftmp = ArgStr(&argc, &argv, "hierAP_SD")) == NULL)
 		goto usage;
-	     if ( strcasecmp(ftmp, "HIER") == 0 )
-		esHierAP = TRUE ;
-	     else if ( strcasecmp(ftmp, "FLAT") == 0 )
-		esHierAP = FALSE ;
+	     if (strcasecmp(ftmp, "HIER") == 0)
+		esHierAP = TRUE;
+	     else if (strcasecmp(ftmp, "FLAT") == 0)
+		esHierAP = FALSE;
 	     else goto usage;
 
 	     break;
@@ -944,7 +945,7 @@ simParseArgs(pargc, pargv)
 
     *pargv = argv;
     *pargc = argc;
-    return 0;
+    return FALSE;
 
 usage:
     TxError("Usage: ext2sim [-a aliasfile] [-A] [-B] [-l labelfile] [-L]\n"
@@ -953,7 +954,7 @@ usage:
 		"[file]\n"
 		);
 
-    return 1;
+    return TRUE;
 }
 
 
@@ -970,9 +971,9 @@ usage:
  * ----------------------------------------------------------------------------
  */
 EFNode *
-SimGetNode(prefix, suffix)
-HierName *prefix;
-HierName *suffix;
+SimGetNode(
+    const HierName *prefix,
+    const HierName *suffix)
 {
 	HashEntry *he;
 
@@ -1011,20 +1012,24 @@ HierName *suffix;
  * ----------------------------------------------------------------------------
  */
 
+/* ARGSUSED */
+/* @typedef cb_extflat_visitdevs_t (UNUSED) */
 int
-simdevVisit(dev, hc, scale, trans)
-    Dev *dev;		/* Device being output */
-    HierContext *hc;	/* Hierarchical context down to this device */
-    float scale;	/* Scale transform for output */
-    Transform *trans;	/* Coordinate transform */
+simdevVisit(
+    Dev *dev,		/* Device being output */
+    HierContext *hc,	/* Hierarchical context down to this device */
+    float scale,	/* Scale transform for output */
+    Transform *trans,	/* Coordinate transform */
+    ClientData cdata)	/* unused */
 {
-    DevTerm *gate, *source, *drain, *term;
-    EFNode  *subnode, *snode, *dnode;
+    const DevTerm *gate, *source, *drain, *term;
+    const EFNode *subnode;
+    EFNode *snode, *dnode;
     int l, w;
     Rect r;
     char name[12];
     bool is_subckt = FALSE;
-    HierName *hierName = hc->hc_hierName;
+    const HierName *hierName = hc->hc_hierName;
 
     sprintf(name, "output");
 
@@ -1122,6 +1127,13 @@ simdevVisit(dev, hc, scale, trans)
 	    fprintf(esSimF, "x");
 	    break;
 
+	case DEV_VERILOGA:
+	    /* Treat verilog-A models as subcircuits (for now, anyway)	*/
+	    /* since IRSIM has no concept of verilog-A models.		*/
+	    is_subckt = TRUE;
+	    fprintf(esSimF, "x");
+	    break;
+
 	default:
 	    fprintf(esSimF, "%c", EFDevTypes[dev->dev_type][0]);
 	    break;
@@ -1261,9 +1273,9 @@ simdevVisit(dev, hc, scale, trans)
 
 	   if (gate->dterm_attrs)
 	       fprintf(esSimF, " g=%s", gate->dterm_attrs);
-	   if ( esFormat == SU ) {
-	       if ( gate->dterm_attrs ) {
-	         subAP = Match(ATTR_SUBSAP, gate->dterm_attrs ) ;
+	   if (esFormat == SU) {
+	       if (gate->dterm_attrs) {
+	         subAP = Match(ATTR_SUBSAP, gate->dterm_attrs);
 	         fprintf(esSimF, ",");
 	       } else
 	         fprintf(esSimF, " g=");
@@ -1272,13 +1284,13 @@ simdevVisit(dev, hc, scale, trans)
 	   }
 	   if (source->dterm_attrs) {
 	       fprintf(esSimF, " s=%s", source->dterm_attrs);
-	       if  ( Match(ATTR_HIERAP, source->dterm_attrs ) != FALSE )
-		   hierS = TRUE ;
-	       else if ( Match(ATTR_FLATAP, source->dterm_attrs ) != FALSE )
-		   hierS = FALSE ;
+	       if (Match(ATTR_HIERAP, source->dterm_attrs) != FALSE)
+		   hierS = TRUE;
+	       else if (Match(ATTR_FLATAP, source->dterm_attrs) != FALSE)
+		   hierS = FALSE;
 	   }
-	   if ( esFormat == SU ) {
-	       fprintf(esSimF, "%s", (source->dterm_attrs) ? "," : " s=" );
+	   if (esFormat == SU) {
+	       fprintf(esSimF, "%s", (source->dterm_attrs) ? "," : " s=");
 	       if (hierS)
 	         simnAPHier(source, hierName, fetInfo[dev->dev_type].resClassSource,
 		      scale, esSimF);
@@ -1290,13 +1302,13 @@ simdevVisit(dev, hc, scale, trans)
 	   }
 	   if (drain->dterm_attrs) {
 	       fprintf(esSimF, " d=%s", drain->dterm_attrs);
-	       if  ( Match(ATTR_HIERAP, drain->dterm_attrs ) != FALSE )
-		   hierD = TRUE ;
-	       else if ( Match(ATTR_FLATAP, drain->dterm_attrs ) != FALSE )
-		   hierD = FALSE ;
+	       if  (Match(ATTR_HIERAP, drain->dterm_attrs) != FALSE)
+		   hierD = TRUE;
+	       else if (Match(ATTR_FLATAP, drain->dterm_attrs) != FALSE)
+		   hierD = FALSE;
 	   }
-	   if ( esFormat == SU ) {
-	       fprintf(esSimF, "%s", (drain->dterm_attrs) ? "," : " d=" );
+	   if (esFormat == SU) {
+	       fprintf(esSimF, "%s", (drain->dterm_attrs) ? "," : " d=");
 	       if (hierD)
 	         simnAPHier(drain, hierName, fetInfo[dev->dev_type].resClassDrain,
 		      scale, esSimF);
@@ -1322,27 +1334,27 @@ simdevVisit(dev, hc, scale, trans)
 }
 
 int
-simdevSubstrate( prefix, suffix, type, scale, doAP, outf)
-HierName *prefix;
-HierName *suffix;
-int type;
-float scale;
-bool doAP;
-FILE *outf;
+simdevSubstrate(
+    const HierName *prefix,
+    const HierName *suffix,
+    int type,
+    float scale,
+    bool doAP,
+    FILE *outf)
 {
     HashEntry *he;
-    EFNodeName *nn;
-    char *suf ;
-    int  l ;
+    const EFNodeName *nn;
+    char *suf;
+    int l;
     EFNode *subnode;
 
     suf = EFHNToStr(suffix);
     if (fetInfo[type].defSubs && strcasecmp(suf,fetInfo[type].defSubs) == 0) {
     	l = strlen(suf) - 1;
-	if (  (( EFOutputFlags & EF_TRIMGLOB ) && suf[l] =='!') ||
-	      (( EFOutputFlags & EF_TRIMLOCAL ) && suf[l] == '#')  )
-	      suf[l] = '\0' ;
-	if ( esFormat == SU )
+	if (((EFOutputFlags & EF_TRIMGLOB) && suf[l] =='!') ||
+	    ((EFOutputFlags & EF_TRIMLOCAL) && suf[l] == '#'))
+	      suf[l] = '\0';
+	if (esFormat == SU)
 		fprintf(outf, "S_");
 	fprintf(outf, "%s", suf);
     }
@@ -1354,11 +1366,11 @@ FILE *outf;
 		return 0;
     	}
     	/* Canonical name */
-    	nn = (EFNodeName *) HashGetValue(he);
+	nn = (const EFNodeName *) HashGetValue(he);
 	subnode = nn->efnn_node;
-	if ( esFormat == SU ) {
-	  if ( doAP ) {
-	    if ( fetInfo[type].resClassSub < 0 ) {
+	if (esFormat == SU) {
+	  if (doAP) {
+	    if (fetInfo[type].resClassSub < 0) {
 	     TxError("Error: subap for devtype %d required but not "
 			"specified on command line\n", type);
 	     fprintf(outf,"A_0,P_0,");
@@ -1392,58 +1404,60 @@ FILE *outf;
  * ----------------------------------------------------------------------------
  */
 
-bool simnAP(node, resClass, scale, outf)
-EFNode *node;
-int  resClass;
-float scale;
-FILE *outf;
+bool
+simnAP(
+    EFNode *node,
+    int resClass,
+    float scale,
+    FILE *outf)
 {
 	int a, p;
 
-	if ( node->efnode_client == (ClientData) NULL )
+	if (node->efnode_client == PTR2CD(NULL))
 		initNodeClient(node);
-	if ( resClass == NO_RESCLASS ||
-	     beenVisited((nodeClient *)node->efnode_client, resClass) ) {
+	if (resClass == NO_RESCLASS ||
+	     beenVisited((nodeClient *)node->efnode_client, resClass)) {
 		fprintf(outf,"A_0,P_0");
 		return FALSE;
 	}
 	markVisited((nodeClient *)node->efnode_client, resClass);
 	a = (int)(node->efnode_pa[resClass].pa_area*scale*scale);
 	p = (int)(node->efnode_pa[resClass].pa_perim*scale);
-	if ( a < 0 ) a = 0;
-	if ( p < 0 ) p = 0;
+	if (a < 0) a = 0;
+	if (p < 0) p = 0;
 	fprintf(outf,"A_%d,P_%d", a, p);
 	return TRUE;
 }
 
-bool simnAPHier(dterm, hierName, resClass, scale, outf)
-    DevTerm *dterm;
-    HierName *hierName;
-    int  resClass;
-    float scale;
-    FILE *outf;
+bool
+simnAPHier(
+    const DevTerm *dterm,
+    const HierName *hierName,
+    int resClass,
+    float scale,
+    FILE *outf)
 {
 	EFNode *node = dterm->dterm_node;
-	nodeClientHier   *nc ;
+	nodeClientHier *nc;
 	int a, p;
 
-	if ( node->efnode_client == (ClientData) NULL )
+	if (node->efnode_client == PTR2CD(NULL))
 		initNodeClientHier(node);
 	nc = (nodeClientHier *)node->efnode_client;
-	if ( nc->lastPrefix != hierName ) {
+	if (nc->lastPrefix != hierName) {
 		TTMaskZero(&(nc->visitMask));
 		nc->lastPrefix = hierName;
 	}
-	if ( resClass == NO_RESCLASS ||
-	     beenVisited((nodeClientHier *)node->efnode_client, resClass) ) {
+	if (resClass == NO_RESCLASS ||
+	    beenVisited((nodeClientHier *)node->efnode_client, resClass)) {
 		fprintf(outf,"A_0,P_0");
 		return FALSE;
 	}
 	markVisited((nodeClientHier *)node->efnode_client, resClass);
 	a = (int)(node->efnode_pa[resClass].pa_area*scale*scale);
 	p = (int)(node->efnode_pa[resClass].pa_perim*scale);
-	if ( a < 0 ) a = 0;
-	if ( p < 0 ) p = 0;
+	if (a < 0) a = 0;
+	if (p < 0) p = 0;
 	fprintf(outf,"A_%d,P_%d", a, p);
 	return TRUE;
 }
@@ -1470,14 +1484,14 @@ bool simnAPHier(dterm, hierName, resClass, scale, outf)
  */
 
 int
-simdevOutNode(prefix, suffix, name, outf)
-    HierName *prefix;
-    HierName *suffix;
-    char *name;
-    FILE *outf;
+simdevOutNode(
+    const HierName *prefix,
+    const HierName *suffix,
+    const char *name,
+    FILE *outf)
 {
     HashEntry *he;
-    EFNodeName *nn;
+    const EFNodeName *nn;
 
     he = EFHNConcatLook(prefix, suffix, name);
     if (he == NULL)
@@ -1487,10 +1501,10 @@ simdevOutNode(prefix, suffix, name, outf)
     }
 
     /* Canonical name */
-    nn = (EFNodeName *) HashGetValue(he);
+    nn = (const EFNodeName *) HashGetValue(he);
     (void) putc(' ', outf);
     EFHNOut(nn->efnn_node->efnode_name->efnn_hier, outf);
-    if ( nn->efnn_node->efnode_client == (ClientData) NULL )
+    if (nn->efnn_node->efnode_client == PTR2CD(NULL))
 	initNodeClient(nn->efnn_node);
     return 0;
 }
@@ -1520,10 +1534,14 @@ simdevOutNode(prefix, suffix, name, outf)
  * ----------------------------------------------------------------------------
  */
 
-int simcapVisit(hierName1, hierName2, cap)
-    HierName *hierName1;
-    HierName *hierName2;
-    double cap;
+/* ARGSUSED */
+/* @typedef cb_extflat_visitcaps_t (UNUSED) */
+int
+simcapVisit(
+    const HierName *hierName1,
+    const HierName *hierName2,
+    double cap,
+    ClientData cdata)	/* unused */
 {
     cap = cap / 1000;
     if (cap <= EFCapThreshold)
@@ -1563,11 +1581,14 @@ int simcapVisit(hierName1, hierName2, cap)
  * ----------------------------------------------------------------------------
  */
 
+/* ARGSUSED */
+/* @typedef cb_extflat_visitresists_t (UNUSED) */
 int
-simresistVisit(hierName1, hierName2, res)
-    HierName *hierName1;
-    HierName *hierName2;
-    float res;
+simresistVisit(
+    const HierName *hierName1,
+    const HierName *hierName2,
+    float res,
+    ClientData cdata)	/* unused */
 {
     fprintf(esSimF, "r ");
     EFHNOut(hierName1, esSimF);
@@ -1595,21 +1616,25 @@ simresistVisit(hierName1, hierName2, res)
  * ----------------------------------------------------------------------------
  */
 
-int simnodeVisit(node, res, cap)
-    EFNode *node;
-    int res;
-    double cap;
+/* ARGSUSED */
+/* @typedef cb_extflat_visitnodes_t (UNUSED) */
+int
+simnodeVisit(
+    EFNode *node,
+    int res,
+    double cap,
+    ClientData cdata) /* unused */
 {
-    EFNodeName *nn;
-    HierName *hierName;
+    const EFNodeName *nn;
+    const HierName *hierName;
     bool isGlob;
     const char *fmt;
-    EFAttr *ap;
+    const EFAttr *ap;
 
-    if (esDevNodesOnly && node->efnode_client == (ClientData) NULL)
+    if (esDevNodesOnly && node->efnode_client == PTR2CD(NULL))
 	return 0;
 
-    hierName = (HierName *) node->efnode_name->efnn_hier;
+    hierName = node->efnode_name->efnn_hier;
     cap = cap  / 1000;
     res = (res + 500) / 1000;
     if (cap > EFCapThreshold)
@@ -1690,13 +1715,18 @@ int simnodeVisit(node, res, cap)
  *
  * ----------------------------------------------------------------------------
  */
-devMerge *simmkDevMerge(l, w, g, s, d, b, hn, dev)
-int   l, w;
-EFNode *g, *s, *d, *b;
-HierName *hn;
-Dev    *dev;
+devMerge *
+simmkDevMerge(
+    int l,
+    int w,
+    const EFNode *g,
+    const EFNode *s,
+    const EFNode *d,
+    const EFNode *b,
+    const HierName *hn,
+    Dev *dev)
 {
-	   devMerge *fp;
+	devMerge *fp;
 
 	fp = (devMerge *) mallocMagic((unsigned) (sizeof(devMerge)));
 	fp->l = l; fp->w = w;
@@ -1733,42 +1763,14 @@ Dev    *dev;
 
 #define parallelDevs(f1, f2) \
 ( \
-	( (f1)->g == (f2)->g && (f1)->b == (f2)->b && (f1)->l == (f2)->l && \
-	  ( esMergeDevsA || (f1)->w == (f2)->w ) )  ? \
-	   ( ((f1)->d == (f2)->d && (f1)->s == (f2)->s ) ? \
+	((f1)->g == (f2)->g && (f1)->b == (f2)->b && (f1)->l == (f2)->l && \
+	  (esMergeDevsA || (f1)->w == (f2)->w))  ? \
+	   (((f1)->d == (f2)->d && (f1)->s == (f2)->s) ? \
 	       PARALLEL : \
-	      (((f1)->s == (f2)->d && (f1)->d == (f2)->s ) ? PARALLEL_R : NOT_PARALLEL) )\
+	      (((f1)->s == (f2)->d && (f1)->d == (f2)->s) ? PARALLEL_R : NOT_PARALLEL)) \
 	  : NOT_PARALLEL  \
 )
 
-/*
- * ----------------------------------------------------------------------------
- *
- * mergeAttr --
- * Macro to merge two attribute strings
- *
- * Results:
- *  The merged strings
- *
- * Side effects:
- *  Might allocate and free memory.
- *
- * ----------------------------------------------------------------------------
- */
-#define	mergeAttr(a1, a2) { \
-	if ( a1 == NULL ) a1 = a2 ; \
-	else { \
-		char *t; \
-		int l1 = strlen(a1); \
-		int l2 = strlen(a2); \
-		t = (char *) mallocMagic((unsigned) (l1+l2)+1); \
-		t = (char *) strcat(a1,a2); \
-		freeMagic(a1); \
-		a1 = t ; \
-	} \
-}
-
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1784,21 +1786,24 @@ Dev    *dev;
  *
  * ----------------------------------------------------------------------------
  */
+
+/* ARGSUSED */
+/* @typedef cb_extflat_visitdevs_t (UNUSED) */
 int
-simmergeVisit(dev, hc, scale, trans)
-Dev *dev;		/* Dev to examine */
-HierContext *hc;	/* Hierarchical context down to this dev */
-float scale;		/* Scale transform */
-Transform *trans;	/* Coordinate transform (not used) */
+simmergeVisit(
+    Dev *dev,		/* Dev to examine */
+    HierContext *hc,	/* Hierarchical context down to this dev */
+    float scale,	/* Scale transform */
+    Transform *trans,	/* Coordinate transform (not used) */
+    ClientData cdata)	/* unused */
 {
-	DevTerm *gate, *source, *drain;
-	Dev     *cf;
-	DevTerm *cg, *cs, *cd;
-	EFNode	*subnode, *snode, *dnode, *gnode;
+	const DevTerm *gate, *source, *drain;
+	const EFNode *subnode, *snode, *dnode, *gnode;
 	int      pmode, l, w;
 	float	 m;
-	devMerge *fp, *cfp;
-	HierName *hierName = hc->hc_hierName;
+	devMerge *fp;
+	const devMerge *cfp;
+	const HierName *hierName = hc->hc_hierName;
 
 	if (dev->dev_nterm < 2) {
 		TxError("outPremature\n");
@@ -1825,24 +1830,25 @@ Transform *trans;	/* Coordinate transform (not used) */
 	 * merge them.
 	 */
 
-	for ( cfp = devMergeList ; cfp != NULL ; cfp = cfp->next ) {
+	for (cfp = devMergeList; cfp != NULL; cfp = cfp->next) {
 	  if ((pmode = parallelDevs(fp, cfp)) != NOT_PARALLEL) {
-
-		cf = cfp->dev;
-		cg = &cfp->dev->dev_terms[0];
-		cs = cd = &cfp->dev->dev_terms[1];
+#if 0 /* -Wunused-but-set-variable cf, cs, cg */
+		Dev *cf = cfp->dev;
+		DevTerm *cg = &cfp->dev->dev_terms[0];
+		DevTerm *cs = &cfp->dev->dev_terms[1];
+		DevTerm *cd = cs;
 		if (cfp->dev->dev_nterm >= 3) {
-			if ( pmode == PARALLEL )
+			if (pmode == PARALLEL)
 				cd = &cfp->dev->dev_terms[2];
-			else if ( pmode == PARALLEL_R )
+			else if (pmode == PARALLEL_R)
 				cs = &cfp->dev->dev_terms[2];
 		}
-
+#endif
 		m = esFMult[cfp->esFMIndex] + ((float)fp->w/(float)cfp->w);
 		setDevMult(fp->esFMIndex, DEV_KILLED);
 		setDevMult(cfp->esFMIndex, m);
 		esDevsMerged++;
-		freeMagic(fp);
+		freeMagic((char *)fp);
 		return 0;
 	  }
 	}
